//
//  AVViewController.m
//  CoverManiac
//
//  Created by Sven Röder on 7/21/12.
//  Copyright (c) 2012 Appventures. All rights reserved.
//

#import "AVViewController.h"
#import <QuartzCore/QuartzCore.h>
#error remember to add your own key for the spotify library
#import "appkey.c"


#define SIZE_OF_ITEM 80
#define FADE_INTERVAL 0.8
#define FAIL_COUNT 3


@interface AVViewController ()
@property (nonatomic, strong) SPPlaybackManager *mngPlayback;
@property (nonatomic, strong) NSMutableArray *availableTracks;
@property (nonatomic, strong) NSTimer *timerFade;
@property int currentFailCount;

- (IBAction)info:(id)sender;
@end


@implementation AVViewController
@synthesize mngPlayback = _mngPlayback;
@synthesize currentFailCount = _currentFailCount;
@synthesize availableTracks = _availableTracks;
@synthesize timerFade = _timerFade;


- (void)viewDidLoad {
    [super viewDidLoad];
	
    [SPSession initializeSharedSessionWithApplicationKey:[NSData dataWithBytes:&g_appkey length:g_appkey_size]
											   userAgent:@"se.appventures.CoverManiac"
										   loadingPolicy:SPAsyncLoadingManual
												   error:nil];
    [[SPSession sharedSession] setDelegate:self];
    [[SPSession sharedSession] setPlaybackDelegate:self];
    
    self.mngPlayback = [[SPPlaybackManager alloc] initWithPlaybackSession:[SPSession sharedSession]];

    [self performSelector:@selector(showLoginView) withObject:nil afterDelay:0.0];
}


#pragma View Helper

- (UIView*)randomCoverView {
    NSMutableArray *buttonViews = [NSMutableArray array];
    
    for (UIView *subview in self.view.subviews) {
        if([subview isKindOfClass:[UIButton class]]) {
            [buttonViews addObject:subview];
        }
    }
    
    if([buttonViews count]) {
        return [buttonViews objectAtIndex:(random() % [buttonViews count])];
    }
    
    return nil;
}


- (void)randomCoverViewFade:(NSTimer*)timer {
    UIView *subview = [self randomCoverView];
    subview.userInteractionEnabled = YES;
    
    UIViewAnimationOptions options = UIViewAnimationOptionAllowUserInteraction | UIViewAnimationOptionBeginFromCurrentState;
    [UIView animateWithDuration:2.0 delay:0.0 options:options animations:^{
        subview.alpha = 1.0;
    } completion:^(BOOL finished) {
        [UIView animateWithDuration:0.5 delay:0.0 options:options animations:^{
            subview.alpha = 0.3;
        } completion:^(BOOL finished) {
            subview.userInteractionEnabled = NO;
        }];
    }];
}


- (UIButton*)coverButtonForTrack:(SPTrack*)track {
    UIButton *btnCover = [UIButton buttonWithType:UIButtonTypeCustom];
    btnCover.alpha = 0.3;
    btnCover.userInteractionEnabled = NO;
    [btnCover setImage:track.album.cover.image forState:UIControlStateNormal];
    [btnCover addTarget:self action:@selector(evaluateChoice:) forControlEvents:UIControlEventTouchUpInside];
    
    return btnCover;
}


- (UIView*)flipsideViewForTrack:(SPTrack*)track frame:(CGRect)frame correct:(BOOL)correct {
    UILabel *lblTitle = [[UILabel alloc] initWithFrame:frame];
    lblTitle.text = track.album.name;
    lblTitle.numberOfLines = 0;
    lblTitle.textAlignment = NSTextAlignmentCenter;
    lblTitle.textColor = [UIColor whiteColor];
    lblTitle.font = [UIFont boldSystemFontOfSize:14];
    lblTitle.backgroundColor = (correct
                                ? [UIColor colorWithRed:0.0 green:1.0 blue:0.0 alpha:0.3]
                                : [UIColor colorWithRed:1.0 green:0.0 blue:0.0 alpha:0.3]);
    [lblTitle sizeThatFits:frame.size];
    
    return lblTitle;
}


- (UIView*)viewForCoverImage:(UIImage*)cover {
    for (UIView *subview in self.view.subviews) {
        if([subview isKindOfClass:[UIButton class]]) {
            if([[(UIButton*)subview imageView].image isEqual:cover]) {
                return subview;
            }
        }
    }
    
    return nil;
}


- (void)clearCovers {
    for (UIView *subview in self.view.subviews) {
        if(![subview isKindOfClass:[UIImageView class]]) {
            [subview removeFromSuperview];
        }
    }
}


- (void)addCoversForTracks:(NSArray*)tracks {
    _availableTracks = [[NSMutableArray alloc] initWithCapacity:tracks.count];
    
    __block int x = 1;
    __block int y = 0;
    
    for (SPTrack *aTrack in tracks) {
        if (aTrack.availability == SP_TRACK_AVAILABILITY_AVAILABLE && [aTrack.name length] > 0) {
            [self.availableTracks addObject:aTrack];
            
            [SPAsyncLoading waitUntilLoaded:aTrack.album.cover timeout:kSPAsyncLoadingDefaultTimeout then:^(NSArray *tracks, NSArray *notLoadedTracks) {
                UIButton *btnCover = [self coverButtonForTrack:aTrack];
                btnCover.frame = CGRectMake(x * SIZE_OF_ITEM, y * SIZE_OF_ITEM, SIZE_OF_ITEM, SIZE_OF_ITEM);
                [self.view addSubview:btnCover];
                
                x++;
                if(x % ((int)self.view.frame.size.width / SIZE_OF_ITEM) == 0) {
                    x = 0;
                    y++;
                }
            }];
        }
    }
}


- (void)transitionFromView:(UIView*)fromView to:(UIView*)toView {
    [UIView transitionFromView:fromView toView:toView duration:1.0 options:UIViewAnimationOptionTransitionFlipFromLeft completion:^(BOOL finished) {
                        [fromView removeFromSuperview];
                        [self playRandomTrack];
                    }];
}


#pragma Data loading, sorting, representation

- (void)playRandomTrack {
    if(!self.availableTracks.count) {
        [self loadCoverData];
    } else {
        SPTrack *randomTrack = [self.availableTracks objectAtIndex:random() % self.availableTracks.count];
        [SPAsyncLoading waitUntilLoaded:randomTrack timeout:kSPAsyncLoadingDefaultTimeout then:^(NSArray *tracks, NSArray *notLoadedTracks) {
            [self.mngPlayback playTrack:randomTrack callback:^(NSError *error) {
                if(!error) {
                    [self.availableTracks removeObject:randomTrack];
                } else {
                    [self playRandomTrack];
                }
            }];
        }];
    }
}


- (void)loadCoverData {
    [self clearCovers];
    
	[SPAsyncLoading waitUntilLoaded:[SPSession sharedSession] timeout:kSPAsyncLoadingDefaultTimeout then:^(NSArray *loadedession, NSArray *notLoadedSession) {
        
		[SPAsyncLoading waitUntilLoaded:[SPSession sharedSession].userPlaylists timeout:kSPAsyncLoadingDefaultTimeout then:^(NSArray *loadedContainers, NSArray *notLoadedContainers) {
            
			NSMutableArray *playlists = [NSMutableArray array];
			[playlists addObject:[SPSession sharedSession].starredPlaylist];
			[playlists addObject:[SPSession sharedSession].inboxPlaylist];
			[playlists addObjectsFromArray:[SPSession sharedSession].userPlaylists.flattenedPlaylists];
			
			[SPAsyncLoading waitUntilLoaded:playlists timeout:kSPAsyncLoadingDefaultTimeout then:^(NSArray *loadedPlaylists, NSArray *notLoadedPlaylists) {
                
				NSArray *playlistItems = [loadedPlaylists valueForKeyPath:@"@unionOfArrays.items"];
				NSArray *tracks = [self tracksFromPlaylistItems:playlistItems];
				
				[SPAsyncLoading waitUntilLoaded:tracks timeout:kSPAsyncLoadingDefaultTimeout then:^(NSArray *loadedTracks, NSArray *notLoadedTracks) {
                    
                    NSInteger max = (self.view.frame.size.width / SIZE_OF_ITEM) * (self.view.frame.size.height / SIZE_OF_ITEM);
                    NSArray *uniquePlaylist = [self filterUniqueAlbumsOfTracks:loadedTracks max:max-1];
                    
                    [self addCoversForTracks:uniquePlaylist];
                    [self playRandomTrack];
                    
                    [self.timerFade invalidate];
                    self.timerFade = nil;
                    self.timerFade = [NSTimer scheduledTimerWithTimeInterval:FADE_INTERVAL
                                                                      target:self
                                                                    selector:@selector(randomCoverViewFade:)
                                                                    userInfo:nil
                                                                     repeats:YES];
				}];
			}];
		}];
	}];
}


- (NSArray*)filterUniqueAlbumsOfTracks:(NSArray*)tracks max:(NSInteger)max {
    NSMutableArray *resultTracks = [NSMutableArray arrayWithCapacity:max];
    NSInteger newMax = (max > tracks.count ? tracks.count : max);
    NSInteger uniqueAlbumsCount = [[NSSet setWithArray:[tracks valueForKey:@"album"]] count];
    
    while (resultTracks.count < newMax) {
        NSSet *albums = [NSSet setWithArray:[resultTracks valueForKey:@"album"]];
        if(albums.count == uniqueAlbumsCount) {
            break;
        }
        
        SPTrack *track = [tracks objectAtIndex:random() % tracks.count];
        if(![albums containsObject:track.album]) {
            [resultTracks addObject:track];
        }
    }
    
    return resultTracks;
}


- (IBAction)evaluateChoice:(UIButton*)sender {
    if([sender.imageView.image isEqual:self.mngPlayback.currentTrack.album.cover.image]) {
        self.currentFailCount = 0;
        [self transitionFromView:sender to:[self flipsideViewForTrack:self.mngPlayback.currentTrack frame:sender.frame correct:YES]];
    } else {
        self.currentFailCount++;
        
        if(self.currentFailCount == FAIL_COUNT) {
            self.currentFailCount = 0;
            
            UIView *fromView = [self viewForCoverImage:self.mngPlayback.currentTrack.album.cover.image];
            [self transitionFromView:fromView to:[self flipsideViewForTrack:self.mngPlayback.currentTrack frame:fromView.frame correct:NO]];
        }
    }
}


- (NSArray*)tracksFromPlaylistItems:(NSArray *)items {
	NSMutableArray *tracks = [NSMutableArray arrayWithCapacity:items.count];
	
	for (SPPlaylistItem *anItem in items) {
		if (anItem.itemClass == [SPTrack class]) {
			[tracks addObject:anItem.item];
		}
	}
	
	return [NSArray arrayWithArray:tracks];
}


#pragma Login

- (void)showLoginView {
	SPLoginViewController *controller = [SPLoginViewController loginControllerForSession:[SPSession sharedSession]];
	controller.allowsCancel = NO;
	controller.loginDelegate = self;
	[self presentViewController:controller animated:NO completion:nil];
}


- (void)sessionDidLoginSuccessfully:(SPSession *)aSession {
    [self loadCoverData];
}


- (void)session:(SPSession *)aSession didFailToLoginWithError:(NSError *)error {
    UIAlertView *alert = [[UIAlertView alloc] initWithTitle:NSLocalizedString(@"Ooops", nil)
                                                    message:[error localizedDescription]
                                                   delegate:self
                                          cancelButtonTitle:NSLocalizedString(@"OK", nil)
                                          otherButtonTitles:nil];
	[alert show];
}


- (void)loginViewController:(SPLoginViewController *)controller didCompleteSuccessfully:(BOOL)didLogin { }


#pragma App Info Popup

- (IBAction)info:(id)sender {
    UIAlertView *alert = [[UIAlertView alloc] initWithTitle:NSLocalizedString(@"Info", nil)
                                                    message:NSLocalizedString(@"Find the album for the current song. The album will flip to green if your choice was right. It will flip to red if your choice was wrong three times in a row. How good do you know your playlists?", nil)
                                                   delegate:self
                                          cancelButtonTitle:NSLocalizedString(@"OK", nil)
                                          otherButtonTitles:nil];
	[alert show];
}


#pragma SessionPlaybackDelegate

- (void)sessionDidEndPlayback:(id<SPSessionPlaybackProvider>)aSession {
    self.currentFailCount = 0;
    UIView *fromView = [self viewForCoverImage:self.mngPlayback.currentTrack.album.cover.image];
    [self transitionFromView:fromView to:[self flipsideViewForTrack:self.mngPlayback.currentTrack frame:fromView.frame correct:NO]];
}


@end
